function independentSubmitFcn(cluster, job, props, ...
    clusterHost, remoteJobStorageLocation, varargin)
%INDEPENDENTSUBMITFCN Submit a MATLAB job to a Slurm cluster
%
% Set your cluster's IndependentSubmitFcn to this function using the following
% command:
%     set(cluster, 'IndependentSubmitFcn', {@independentSubmitFcn, clusterHost, remoteJobStorageLocation});
%
% See also parallel.cluster.generic.independentDecodeFcn.
%

% Copyright 2010-2016 The MathWorks, Inc.
% (modifications by SH@UOL Jan 2017 for CARL/EDDY clusters)

%%%%%%% parse extra arguments %%%%%%%%%%%%% 
%  
% (based on code from RL@UOL in 2013)
% - added varargin to parameter list
% - created p as inputParser and added parameters for SLURM submission
% - added extra parameters to creation of command file
%
p = inputParser;

% possible parameters can be added here as default
defaultPartition = '';
defaultRuntime = '24:0:0';
defaultMemory = '2G';
defaultDiskspace='50M';
defaultMailUser='';
defaultMailType='ALL';

% possible parameters can be added here for parsing
addParameter(p, 'partition', defaultPartition, @ischar); 
addParameter(p, 'runtime', defaultRuntime, @ischar);
addParameter(p, 'memory', defaultMemory, @ischar);
addParameter(p, 'diskspace', defaultDiskspace, @ischar);
addParameter(p, 'mailuser', defaultMailUser, @ischar);
addParameter(p, 'mailtype', defaultMailType, @ischar);

parse(p, varargin{:});

% output
if strcmp(p.Results.partition, defaultPartition)
  disp(['partition    = NONE selected (default)'])
else
  disp(['partition    = ', p.Results.partition])
end

if strcmp(p.Results.runtime, defaultRuntime)
  disp(['runtime      = ', p.Results.runtime, ' (default)'])
else
  disp(['runtime      = ', p.Results.runtime])
end

if strcmp(p.Results.memory, defaultMemory)
  disp(['memory       = ', p.Results.memory, ' (default)'])
else
  disp(['memory       = ', p.Results.memory])
end

if strcmp(p.Results.diskspace, defaultDiskspace)
  disp(['diskspace    = ', p.Results.diskspace, ' (default)'])
else
  disp(['diskspace    = ', p.Results.diskspace])
end

pattern='[A-Z0-9._%+-]{1,64}@(?:[A-Z0-9-]+\.){1,8}[A-Z]{2,63}';
if ~strcmp(p.Results.mailuser,regexpi(p.Results.mailuser,pattern,'match'))
  disp 'Cannot validate e-mail address in mailuser option, mailuser will not be used'
  p.Results.mailuser=''
end

if strcmp(p.Results.mailuser, defaultMailUser)
  disp(['mailuser     = NONE (default)'])
  disp(['mailtype     = NONE (default)'])
else
  disp(['mailuser     = ', p.Results.mailuser])
  if strcmp(p.Results.mailtype, defaultMailType)
    disp(['mailtype     = ', p.Results.mailtype, ' (default)'])
  else
	disp(['mailtype     = ', p.Results.mailtype])
  end
end
%
%%%%%%% end of extra arguments %%%%%%%%%%%%

% Store the current filename for the errors, warnings and dctSchedulerMessages
currFilename = mfilename;
if ~isa(cluster, 'parallel.Cluster')
    error('parallelexamples:GenericSLURM:SubmitFcnError', ...
        'The function %s is for use with clusters created using the parcluster command.', currFilename)
end

decodeFunction = 'parallel.cluster.generic.independentDecodeFcn';

if cluster.HasSharedFilesystem
    error('parallelexamples:GenericSLURM:SubmitFcnError', ...
        'The submit function %s is for use with nonshared filesystems.', currFilename)
end

if ~strcmpi(cluster.OperatingSystem, 'unix')
    error('parallelexamples:GenericSLURM:SubmitFcnError', ...
        'The submit function %s only supports clusters with unix OS.', currFilename)
end
if ~ischar(clusterHost)
    error('parallelexamples:GenericSLURM:IncorrectArguments', ...
        'Hostname must be a string');
end
if ~ischar(remoteJobStorageLocation)
    error('parallelexamples:GenericSLURM:IncorrectArguments', ...
        'Remote job storage location must be a string');
end

remoteConnection = getRemoteConnection(cluster, clusterHost, remoteJobStorageLocation);

% The job specific environment variables
% Remove leading and trailing whitespace from the MATLAB arguments
matlabArguments = strtrim(props.MatlabArguments);
variables = {'MDCE_DECODE_FUNCTION', decodeFunction; ...
    'MDCE_STORAGE_CONSTRUCTOR', props.StorageConstructor; ...
    'MDCE_JOB_LOCATION', props.JobLocation; ...
    'MDCE_MATLAB_EXE', props.MatlabExecutable; ...
    'MDCE_MATLAB_ARGS', matlabArguments; ...
    'MDCE_DEBUG', 'true'; ...
    'MLM_WEB_LICENSE', props.UseMathworksHostedLicensing; ...
    'MLM_WEB_USER_CRED', props.UserToken; ...
    'MLM_WEB_ID', props.LicenseWebID; ...
    'MDCE_LICENSE_NUMBER', props.LicenseNumber; ...
    'MDCE_STORAGE_LOCATION', remoteJobStorageLocation};
% Trim the environment variables of empty values.
nonEmptyValues = cellfun(@(x) ~isempty(strtrim(x)), variables(:,2));
variables = variables(nonEmptyValues, :);

% Get the correct quote and file separator for the Cluster OS.
% This check is unnecessary in this file because we explicitly
% checked that the ClusterOsType is unix.  This code is an example
% of how your integration code should deal with clusters that
% can be unix or pc.
if strcmpi(cluster.OperatingSystem, 'unix')
    quote = '''';
    fileSeparator = '/';
else
    quote = '"';
    fileSeparator = '\';
end

% The local job directory
localJobDirectory = cluster.getJobFolder(job);
% How we refer to the job directory on the cluster
remoteJobDirectory = remoteConnection.getRemoteJobLocation(job.ID, cluster.OperatingSystem);

% The script name is independentJobWrapper.sh
scriptName = 'independentJobWrapper.sh';
% The wrapper script is in the same directory as this file
dirpart = fileparts(mfilename('fullpath'));
localScript = fullfile(dirpart, scriptName);
% Copy the local wrapper script to the job directory
copyfile(localScript, localJobDirectory);

% The command that will be executed on the remote host to run the job.
remoteScriptName = sprintf('%s%s%s', remoteJobDirectory, fileSeparator, scriptName);
quotedScriptName = sprintf('%s%s%s', quote, remoteScriptName, quote);

% Get the tasks for use in the loop
tasks = job.Tasks;
numberOfTasks = props.NumberOfTasks;
jobIDs = cell(numberOfTasks, 1);
commandsToRun = cell(numberOfTasks, 1);
% Loop over every task we have been asked to submit
for ii = 1:numberOfTasks
    taskLocation = props.TaskLocations{ii};
    % Add the task location to the environment variables
    environmentVariables = [variables; ...
        {'MDCE_TASK_LOCATION', taskLocation}];
    
    % Choose a file for the output. Please note that currently, JobStorageLocation refers
    % to a directory on disk, but this may change in the future.
    logFile = sprintf('%s%s%s', remoteJobDirectory, fileSeparator, sprintf('Task%d.log', tasks(ii).ID));
    quotedLogFile = sprintf('%s%s%s', quote, logFile, quote);
    
    % Submit one task at a time
    jobName = sprintf('Job%d.%d', job.ID, tasks(ii).ID);
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %% CUSTOMIZATION MAY BE REQUIRED %%
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % You may also wish to supply additional submission arguments to
    % the sbatch command here.
    additionalSubmitArgs = '--ntasks=1 --licenses=mdcs:1 ';
    additionalSubmitArgs = [ additionalSubmitArgs ...
    sprintf('--mem-per-cpu=%s ', p.Results.memory) ...
    sprintf('--time=%s ', p.Results.runtime) ...
    sprintf('--gres=tmpdir:%s ', p.Results.diskspace) ...
    ];
    % optional arguments that are only added if set
    if (~isempty(p.Results.partition))
        additionalSubmitArgs = [ additionalSubmitArgs ...
             sprintf('--partition=%s ', p.Results.partition) ...
             ];
    end
    if (~isempty(p.Results.mailuser))
        additionalSubmitArgs = [ additionalSubmitArgs ...
             sprintf('--mail-user=%s ', p.Results.mailuser) ...
             sprintf('--mail-type=%s ', p.Results.mailtype) ...
	     ];
    end

    % Create a script to submit a Slurm job - this will be created in the job directory
    dctSchedulerMessage(5, '%s: Generating script for task %i', currFilename, ii);
    localScriptName = tempname(localJobDirectory);
    [~, scriptName] = fileparts(localScriptName);
    remoteScriptLocation = sprintf('%s%s%s', remoteJobDirectory, fileSeparator, scriptName);
    createSubmitScript(localScriptName, jobName, quotedLogFile, quotedScriptName, ...
        environmentVariables, additionalSubmitArgs);
    % Create the command to run on the remote host.
    commandsToRun{ii} = sprintf('sh %s', remoteScriptLocation);
end
dctSchedulerMessage(4, '%s: Starting mirror for job %d.', currFilename, job.ID);
% Start the mirror to copy all the job files over to the cluster
remoteConnection.startMirrorForJob(job);
for ii = 1:numberOfTasks
    commandToRun = commandsToRun{ii};
    
    % Now ask the cluster to run the submission command
    dctSchedulerMessage(4, '%s: Submitting job using command:\n\t%s', currFilename, commandToRun);
    % Execute the command on the remote host.
    [cmdFailed, cmdOut] = remoteConnection.runCommand(commandToRun);
    if cmdFailed
        % Stop the mirroring if we failed to submit the job - this will also
        % remove the job files from the remote location
        % Only stop mirroring if we are actually mirroring
        if remoteConnection.isJobUsingConnection(job.ID)
            dctSchedulerMessage(5, '%s: Stopping the mirror for job %d.', currFilename, job.ID);
            try
                remoteConnection.stopMirrorForJob(job);
            catch err
                warning('parallelexamples:GenericSLURM:FailedToStopMirrorForJob', ...
                    'Failed to stop the file mirroring for job %d.\nReason: %s', ...
                    job.ID, err.getReport);
            end
        end
        error('parallelexamples:GenericSLURM:FailedToSubmitJob', ...
            'Failed to submit job to Slurm using command:\n\t%s.\nReason: %s', ...
            commandToRun, cmdOut);
    end
    jobIDs{ii} = extractJobId(cmdOut);
    
    if isempty(jobIDs{ii})
        warning('parallelexamples:GenericSLURM:FailedToParseSubmissionOutput', ...
            'Failed to parse the job identifier from the submission output: "%s"', ...
            cmdOut);
    end
end

% set the cluster host, remote job storage location and job ID on the job cluster data
jobData = struct('ClusterJobIDs', {jobIDs}, ...
    'RemoteHost', clusterHost, ...
    'RemoteJobStorageLocation', remoteJobStorageLocation, ...
    'HasDoneLastMirror', false);
cluster.setJobClusterData(job, jobData);
